@@ -1,10 +1,10 @@
 #                              -*- Mode: Perl -*- 
 # $Basename: Screen.pm $
-# $Revision: 1.25 $
+# $Revision: 1.28 $
 # Author          : Ulrich Pfeifer
 # Created On      : Thu Dec 18 09:26:31 1997
 # Last Modified By: Ulrich Pfeifer
-# Last Modified On: Fri Jul 10 21:52:31 1998
+# Last Modified On: Mon Feb 14 17:30:32 2000
 # Language        : CPerl
 # 
 # (C) Copyright 1997, Ulrich Pfeifer
@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 
 # $Format: "$\VERSION = sprintf '%5.3f', ($ProjectMajorVersion$ * 100 + ($ProjectMinorVersion$-1))/1000;"$
-$VERSION = sprintf '%5.3f', (1 * 100 + (17-1))/1000;
+$VERSION = sprintf '%5.3f', (1 * 100 + (20-1))/1000;
 
 sub _set_screen {
   my ($self, $screen, $title)  = @_;
@@ -38,7 +38,10 @@ sub _set_screen {
   if ( @screen_last_name > 1 and $screen_last_name[-2] eq $screen) {
     # User did hit a 'back' button
     pop @screen_last_name; pop @screen_last_title;
-  } elsif (@screen_last_name and $screen_last_name[-1] ne $screen) {
+  } elsif (not $self->{dont_cut_loops} and
+           @screen_last_name and $screen_last_name[-1] eq $screen) {
+    # Do nothing. We did jump to the same screen again
+  } else {
     while (@screen_last_name > 7) {shift @screen_last_name; shift @screen_last_title;}
     push @screen_last_name,  $self->{screen_name};
     push @screen_last_title, $self->{screen_title};
@@ -55,7 +58,7 @@ sub last_screen {
   my $self = shift;
   
   if (wantarray) {
-    ($self->{screen_last_name}, $self->{screen_last_title});
+    ($self->{screen_last_name}, $self->{screen_last_title}||$self->{screen_last_name});
   } else {
     $self->{screen_last_name};
   }
@@ -96,7 +99,7 @@ sub new {
   }
 
   for ($self->{cgi}->param) {      # hunt for the target screen
-    if (/^screen_(.*)$/) {
+    if (/^screen_function(.*)$/) {
       if ($self->_set_screen($1)) {
         $self->{cgi}->delete($_);
         return $self;              # shortcut the rest
@@ -137,7 +140,8 @@ sub import {
         if (@_ and $_[0] eq '-name') {
           $self->{passed}->{$_[1]} = 1;
         }
-        &{"CGI::$sym"}($self->{cgi}, @_);
+        #warn "CGI $sym(@_)\n";
+        &{"CGI::$sym"}(@_);
       }
     }
   }
@@ -156,6 +160,7 @@ sub AUTOLOAD {
       if (@_>1 and $_[0] eq '-name') {
         $self->{passed}->{$_[1]} = 1;
       }
+      #warn "AL $func(@_)\n";
       &$code($self->{cgi}, @_);
     };
     goto &$func;
@@ -190,11 +195,12 @@ sub dispatch {
 }
 
 sub application { 'CGI::Screen Test' }
+sub default_title { 'Welcome' }
 
 sub title {
   my ($query, $screen, $title)  = @_;
 
-  $query->application . ': ' . ($title || $screen);
+  $query->application . ': ' . ($title || $screen || $query->default_title);
 }
 
 sub headline {
@@ -206,10 +212,10 @@ sub headline {
 sub prologue {
   my $query  = shift;
   my $screen = $query->{'screen_name'};
-  my $title  = $query->{'screen_title'};
+  my $title  = $query->{'screen_title'} || $query->default_title;
   print
-    $query->header    ('-type'  => 'text/html'),
-    $query->start_html('-title' => $query->title($title)),
+    $query->header    ('-type'  => 'text/html', -expires => '+10s'),
+    $query->start_html('-title' => $query->title($screen, $title)),
     $query->_call('headline'),
     $query->start_form
       ;
@@ -259,7 +265,7 @@ sub goto_screen {
 
   $query->submit
     (
-     -name  => 'screen_' . $screen,
+     -name  => 'screen_function' . $screen,
      -value => $name,
     );
 }
@@ -272,7 +278,7 @@ sub url_to_screen {
                                              # announce this function
                                              # ;-(
   
-  $url .= $escape->('screen_' . $screen) . '=' . $escape->($title||'');
+  $url .= $escape->('screen_function' . $screen) . '=' . $escape->($title||'');
   for my $param ($query->param) {
     next if exists $parm{$param};
     for my $value ($query->param($param)) {
@@ -347,7 +353,7 @@ __END__
 
 =head1 NAME
 
-CGI::Screen - Perl extension for easy creation of multi screen CGI-scripts
+CGI::Screen - Perl extension for easy creation of multi screen CGI scripts
 
 =head1 SYNOPSIS
 
@@ -366,18 +372,18 @@ This is B<alpha> software. User visible changes can happen any time.
 =head1 DESCRIPTION
 
 B<CGI::Screen> is a subclass of C<CGI> which allows the esay(TM)
-creation of simple multi screen CGI-Scripts. By 'multi screen' I mean
+creation of simple multi screen CGI scripts. By 'multi screen' I mean
 scripts which present different screens to the user when called with
 different parameters. This is the common case for scripts linking to
 themselves.
 
 To use B<CGI::Screen>, you have to subclass it. For each screen you
 want to present to the user, you must create a method
-C<screen_>I<screen_name>. This method has to produce the HTML code for
+I<screen_name>C<screen_>. This method has to produce the HTML code for
 the screen. CGI::Screen does generate HTTP headers and an HTML
-framework for you. The HTML-framework already contains the C<FORM>
-tags.  You can customize the HTTP headers HTML framework by providing
-callback methods.
+framework for you. The HTML framework already contains the C<FORM>
+tags.  You can customize the HTTP headers and the HTML framework by 
+providing callback methods.
 
 CGI::Screen keeps track of the CGI parameters used in your screen and
 passes old parameters which are not used in the current screen.
@@ -387,7 +393,7 @@ screens the script implements. The C<screen_user> and C<screen_passwd>
 fields are used if you enable the builtin simple authentication.  In
 general you should advice your HTTP server to do authentication. But
 sometimes it is convenient to check the authentication at the script
-level. Especially if you do not have access to yours servers
+level. Especially if you do not have access to your server's
 configuration.
 
 =head2 The constructor C<new>
@@ -395,7 +401,21 @@ configuration.
 If the first parameter of C<new> is the string C<-screen> the second
 argument must be a hash reference specifying the options for the
 subclass. Other parameters are passed to the constructor of C<CGI>.
-Currently no options are used.
+
+=over
+
+=item C<-dont_cut_loops>
+
+Normaly the history of pages will not be extended if the current page is
+the same as the last page. So looping on a page will not change the
+result of the C<last_screen> method. If the option C<-dont_cut_loops>
+is provided and true, the page will recorded twice. A third visit will
+be interpreted as jump back to the first visit.
+
+I<That sounds weird. Will have to figure out a way to recognize back
+jumps independent of the history.>
+
+=back
 
 =head2 Adding Screens
 
@@ -403,8 +423,8 @@ All applications should provide a B<main> screen by defining a method
 C<main_screen>. This method is called if no (existing) screen is
 specified in the parameters. The method is called with three
 arguments: The query object, the screen name and the screen title
-(More precisely the third parameter (if present) is the text on the
-button or anchor which cause the jump to this page).
+(More precisely the third parameter, if present, is the text on the
+button or anchor which caused the jump to this page).
 
 So the minimal application looks like this:
 
@@ -422,7 +442,7 @@ So the minimal application looks like this:
     print $query->p('This is the Main Screen');
   }
 
-That is not too exiting. Let us add a second screen and allow
+That is not too exciting. Let us add a second screen and allow
 navigation between the screens:
 
   sub main_screen {
@@ -449,6 +469,7 @@ another screen.  You can also produce an anchor instead of a button by
 calling C<link_to_screen> instead of C<goto_screen>. You may pass
 additional parameters to encode:
 
+    my %score = some_fancy_search_algorithm($self->param('query'));
     for my $docid (keys %score) {
       print $query->link_to_screen('display', $title{$docid},
                                    'docid' => $docid,
@@ -540,7 +561,8 @@ to start your screens.
 
   sub headline { $_[0]->h1(title(@_)) }
 
-You should overwrite the C<application> method if you use the default title and headline.
+You should overwrite the C<application> method if you use the default
+title and headline.
 
   sub application { 'CGI::Screen Test' }
 
@@ -565,10 +587,10 @@ C<new_form>.
      print
        $query->p('This is the Main Screen'),
        $query->textfield('foo'),
-       $query->goto('First'),
+       $query->goto_screen('first', 'First'),
        $query->new_form,
        $query->textfield('foo'),
-       $query->goto('Second');
+       $query->goto_screen('second', 'Second');
   }
 
 =head2 Non HTML screens
@@ -627,11 +649,11 @@ distribution.
 =head1 ACKNOWLEDGEMENTS
 
 I wish to thank Andreas Koenig F<koenig@kulturbox.de> for the
-fruitfully discussion about the design of this module. 
+fruitful discussion about the design of this module. 
 
 =head1 Copyright
 
-The B<CGI::Screen> module is Copyright (c) 1997 Ulrich
+The B<CGI::Screen> module is Copyright (c) 1997,1998 Ulrich
 Pfeifer. Germany.  All rights reserved.
 
 You may distribute under the terms of either the GNU General Public
